@@ -1,8 +1,8 @@
 """TiTiler.mosaic Router factories."""
 
 import os
-from dataclasses import dataclass, field
-from typing import Callable, Dict, Optional, Type, Union
+from dataclasses import dataclass
+from typing import Callable, Dict, List, Literal, Optional, Tuple, Type, Union
 from urllib.parse import urlencode
 
 import rasterio
@@ -11,24 +11,40 @@ from cogeo_mosaic.models import Info as mosaicInfo
 from cogeo_mosaic.mosaic import MosaicJSON
 from geojson_pydantic.features import Feature
 from geojson_pydantic.geometries import Polygon
-from morecantile import TileMatrixSet
+from morecantile import tms
+from morecantile.defaults import TileMatrixSets
 from rio_tiler.constants import MAX_THREADS
-from rio_tiler.io import BaseReader, COGReader, MultiBandReader, MultiBaseReader
+from rio_tiler.io import BaseReader, MultiBandReader, MultiBaseReader, Reader
 from rio_tiler.models import Bounds
+from rio_tiler.mosaic.methods.base import MosaicMethodBase
 
-from titiler.core.dependencies import WebMercatorTMSParams
+from titiler.core.dependencies import DefaultDependency, RescalingParams
 from titiler.core.factory import BaseTilerFactory, img_endpoint_params, templates
 from titiler.core.models.mapbox import TileJSON
 from titiler.core.resources.enums import ImageType, MediaType, OptionalHeader
 from titiler.core.resources.responses import GeoJSONResponse, JSONResponse, XMLResponse
-from titiler.core.utils import Timer
 from titiler.mosaic.models.responses import Point
 from titiler.mosaic.resources.enums import PixelSelectionMethod
 
 from fastapi import Depends, Path, Query
 
 from starlette.requests import Request
-from starlette.responses import Response
+from starlette.responses import HTMLResponse, Response
+
+# BaseBackend does not support other TMS than WebMercator
+default_tms = TileMatrixSets({"WebMercatorQuad": tms.get("WebMercatorQuad")})
+
+
+def PixelSelectionParams(
+    pixel_selection: PixelSelectionMethod = Query(
+        PixelSelectionMethod.first,
+        description="Pixel selection method.",
+    )
+) -> MosaicMethodBase:
+    """
+    Returns the mosaic method used to combine datasets together.
+    """
+    return pixel_selection.method()
 
 
 @dataclass
@@ -45,12 +61,16 @@ class MosaicTilerFactory(BaseTilerFactory):
         Type[BaseReader],
         Type[MultiBaseReader],
         Type[MultiBandReader],
-    ] = COGReader
+    ] = Reader
 
-    # BaseBackend does not support other TMS than WebMercator
-    tms_dependency: Callable[..., TileMatrixSet] = WebMercatorTMSParams
+    backend_dependency: Type[DefaultDependency] = DefaultDependency
 
-    backend_options: Dict = field(default_factory=dict)
+    pixel_selection_dependency: Callable[..., MosaicMethodBase] = PixelSelectionParams
+
+    supported_tms: TileMatrixSets = default_tms
+
+    # Add/Remove some endpoints
+    add_viewer: bool = True
 
     def register_routes(self):
         """
@@ -72,6 +92,10 @@ class MosaicTilerFactory(BaseTilerFactory):
         self.validate()
         self.assets()
 
+        # Optional Routes
+        if self.add_viewer:
+            self.map_viewer()
+
     ############################################################################
     # /read
     ############################################################################
@@ -91,13 +115,19 @@ class MosaicTilerFactory(BaseTilerFactory):
             response_model_exclude_none=True,
             responses={200: {"description": "Return MosaicJSON definition"}},
         )
-        def read(src_path=Depends(self.path_dependency)):
+        def read(
+            src_path=Depends(self.path_dependency),
+            backend_params=Depends(self.backend_dependency),
+            reader_params=Depends(self.reader_dependency),
+            env=Depends(self.environment_dependency),
+        ):
             """Read a MosaicJSON"""
-            with rasterio.Env(**self.gdal_config):
+            with rasterio.Env(**env):
                 with self.reader(
                     src_path,
                     reader=self.dataset_reader,
-                    **self.backend_options,
+                    reader_options={**reader_params},
+                    **backend_params,
                 ) as src_dst:
                     return src_dst.mosaic_def
 
@@ -112,13 +142,19 @@ class MosaicTilerFactory(BaseTilerFactory):
             response_model=Bounds,
             responses={200: {"description": "Return the bounds of the MosaicJSON"}},
         )
-        def bounds(src_path=Depends(self.path_dependency)):
-            """Return the bounds of the COG."""
-            with rasterio.Env(**self.gdal_config):
+        def bounds(
+            src_path=Depends(self.path_dependency),
+            backend_params=Depends(self.backend_dependency),
+            reader_params=Depends(self.reader_dependency),
+            env=Depends(self.environment_dependency),
+        ):
+            """Return the bounds of the MosaicJSON."""
+            with rasterio.Env(**env):
                 with self.reader(
                     src_path,
                     reader=self.dataset_reader,
-                    **self.backend_options,
+                    reader_options={**reader_params},
+                    **backend_params,
                 ) as src_dst:
                     return {"bounds": src_dst.geographic_bounds}
 
@@ -133,13 +169,19 @@ class MosaicTilerFactory(BaseTilerFactory):
             response_model=mosaicInfo,
             responses={200: {"description": "Return info about the MosaicJSON"}},
         )
-        def info(src_path=Depends(self.path_dependency)):
+        def info(
+            src_path=Depends(self.path_dependency),
+            backend_params=Depends(self.backend_dependency),
+            reader_params=Depends(self.reader_dependency),
+            env=Depends(self.environment_dependency),
+        ):
             """Return basic info."""
-            with rasterio.Env(**self.gdal_config):
+            with rasterio.Env(**env):
                 with self.reader(
                     src_path,
                     reader=self.dataset_reader,
-                    **self.backend_options,
+                    reader_options={**reader_params},
+                    **backend_params,
                 ) as src_dst:
                     return src_dst.info()
 
@@ -155,13 +197,19 @@ class MosaicTilerFactory(BaseTilerFactory):
                 }
             },
         )
-        def info_geojson(src_path=Depends(self.path_dependency)):
+        def info_geojson(
+            src_path=Depends(self.path_dependency),
+            backend_params=Depends(self.backend_dependency),
+            reader_params=Depends(self.reader_dependency),
+            env=Depends(self.environment_dependency),
+        ):
             """Return mosaic's basic info as a GeoJSON feature."""
-            with rasterio.Env(**self.gdal_config):
+            with rasterio.Env(**env):
                 with self.reader(
                     src_path,
                     reader=self.dataset_reader,
-                    **self.backend_options,
+                    reader_options={**reader_params},
+                    **backend_params,
                 ) as src_dst:
                     info = src_dst.info()
                     return Feature(
@@ -193,7 +241,10 @@ class MosaicTilerFactory(BaseTilerFactory):
             z: int = Path(..., ge=0, le=30, description="Mercator tiles's zoom level"),
             x: int = Path(..., description="Mercator tiles's column"),
             y: int = Path(..., description="Mercator tiles's row"),
-            tms: TileMatrixSet = Depends(self.tms_dependency),
+            TileMatrixSetId: Literal[tuple(self.supported_tms.list())] = Query(
+                "WebMercatorQuad",
+                description="TileMatrixSet Name (default: 'WebMercatorQuad')",
+            ),  # noqa
             scale: int = Query(
                 1, gt=0, lt=4, description="Tile size scale. 1=256x256, 2=512x512..."
             ),
@@ -203,73 +254,70 @@ class MosaicTilerFactory(BaseTilerFactory):
             src_path=Depends(self.path_dependency),
             layer_params=Depends(self.layer_dependency),
             dataset_params=Depends(self.dataset_dependency),
-            pixel_selection: PixelSelectionMethod = Query(
-                PixelSelectionMethod.first, description="Pixel selection method."
-            ),
-            postprocess_params=Depends(self.process_dependency),
-            colormap=Depends(self.colormap_dependency),
-            render_params=Depends(self.render_dependency),
-            tile_buffer: Optional[float] = Query(
+            pixel_selection=Depends(self.pixel_selection_dependency),
+            buffer: Optional[float] = Query(
                 None,
                 gt=0,
-                alias="buffer",
                 title="Tile buffer.",
                 description="Buffer on each side of the given tile. It must be a multiple of `0.5`. Output **tilesize** will be expanded to `tilesize + 2 * tile_buffer` (e.g 0.5 = 257x257, 1.0 = 258x258).",
             ),
+            post_process=Depends(self.process_dependency),
+            rescale: Optional[List[Tuple[float, ...]]] = Depends(RescalingParams),
+            color_formula: Optional[str] = Query(
+                None,
+                title="Color Formula",
+                description="rio-color formula (info: https://github.com/mapbox/rio-color)",
+            ),
+            colormap=Depends(self.colormap_dependency),
+            render_params=Depends(self.render_dependency),
+            backend_params=Depends(self.backend_dependency),
+            reader_params=Depends(self.reader_dependency),
+            env=Depends(self.environment_dependency),
         ):
             """Create map tile from a COG."""
-            timings = []
-            headers: Dict[str, str] = {}
-
-            tilesize = scale * 256
-
             threads = int(os.getenv("MOSAIC_CONCURRENCY", MAX_THREADS))
-            with Timer() as t:
-                with rasterio.Env(**self.gdal_config):
-                    with self.reader(
-                        src_path,
-                        reader=self.dataset_reader,
-                        **self.backend_options,
-                    ) as src_dst:
-                        mosaic_read = t.from_start
-                        timings.append(("mosaicread", round(mosaic_read * 1000, 2)))
 
-                        data, _ = src_dst.tile(
-                            x,
-                            y,
-                            z,
-                            pixel_selection=pixel_selection.method(),
-                            tilesize=tilesize,
-                            threads=threads,
-                            tile_buffer=tile_buffer,
-                            **layer_params,
-                            **dataset_params,
-                        )
-            timings.append(("dataread", round((t.elapsed - mosaic_read) * 1000, 2)))
+            with rasterio.Env(**env):
+                with self.reader(
+                    src_path,
+                    reader=self.dataset_reader,
+                    reader_options={**reader_params},
+                    **backend_params,
+                ) as src_dst:
+                    image, assets = src_dst.tile(
+                        x,
+                        y,
+                        z,
+                        pixel_selection=pixel_selection,
+                        tilesize=scale * 256,
+                        threads=threads,
+                        buffer=buffer,
+                        **layer_params,
+                        **dataset_params,
+                    )
+
+            if post_process:
+                image = post_process(image)
+
+            if rescale:
+                image.rescale(rescale)
+
+            if color_formula:
+                image.apply_color_formula(color_formula)
 
             if not format:
-                format = ImageType.jpeg if data.mask.all() else ImageType.png
+                format = ImageType.jpeg if image.mask.all() else ImageType.png
 
-            with Timer() as t:
-                image = data.post_process(**postprocess_params)
-            timings.append(("postprocess", round(t.elapsed * 1000, 2)))
-
-            with Timer() as t:
-                content = image.render(
-                    img_format=format.driver,
-                    colormap=colormap,
-                    **format.profile,
-                    **render_params,
-                )
-            timings.append(("format", round(t.elapsed * 1000, 2)))
-
-            if OptionalHeader.server_timing in self.optional_headers:
-                headers["Server-Timing"] = ", ".join(
-                    [f"{name};dur={time}" for (name, time) in timings]
-                )
+            content = image.render(
+                img_format=format.driver,
+                colormap=colormap,
+                **format.profile,
+                **render_params,
+            )
 
+            headers: Dict[str, str] = {}
             if OptionalHeader.x_assets in self.optional_headers:
-                headers["X-Assets"] = ",".join(data.assets)
+                headers["X-Assets"] = ",".join(assets)
 
             return Response(content, media_type=format.mediatype, headers=headers)
 
@@ -290,7 +338,10 @@ class MosaicTilerFactory(BaseTilerFactory):
         )
         def tilejson(
             request: Request,
-            tms: TileMatrixSet = Depends(self.tms_dependency),
+            TileMatrixSetId: Literal[tuple(self.supported_tms.list())] = Query(
+                "WebMercatorQuad",
+                description="TileMatrixSet Name (default: 'WebMercatorQuad')",
+            ),  # noqa
             src_path=Depends(self.path_dependency),
             tile_format: Optional[ImageType] = Query(
                 None, description="Output image type. Default is auto."
@@ -306,19 +357,27 @@ class MosaicTilerFactory(BaseTilerFactory):
             ),
             layer_params=Depends(self.layer_dependency),  # noqa
             dataset_params=Depends(self.dataset_dependency),  # noqa
-            pixel_selection: PixelSelectionMethod = Query(
-                PixelSelectionMethod.first, description="Pixel selection method."
-            ),  # noqa
-            postprocess_params=Depends(self.process_dependency),  # noqa
-            colormap=Depends(self.colormap_dependency),  # noqa
-            render_params=Depends(self.render_dependency),  # noqa
-            tile_buffer: Optional[float] = Query(  # noqa
+            pixel_selection=Depends(self.pixel_selection_dependency),  # noqa
+            buffer: Optional[float] = Query(  # noqa
                 None,
                 gt=0,
-                alias="buffer",
                 title="Tile buffer.",
                 description="Buffer on each side of the given tile. It must be a multiple of `0.5`. Output **tilesize** will be expanded to `tilesize + 2 * tile_buffer` (e.g 0.5 = 257x257, 1.0 = 258x258).",
             ),
+            post_process=Depends(self.process_dependency),  # noqa
+            rescale: Optional[List[Tuple[float, ...]]] = Depends(
+                RescalingParams
+            ),  # noqa
+            color_formula: Optional[str] = Query(  # noqa
+                None,
+                title="Color Formula",
+                description="rio-color formula (info: https://github.com/mapbox/rio-color)",
+            ),
+            colormap=Depends(self.colormap_dependency),  # noqa
+            render_params=Depends(self.render_dependency),  # noqa
+            backend_params=Depends(self.backend_dependency),
+            reader_params=Depends(self.reader_dependency),
+            env=Depends(self.environment_dependency),
         ):
             """Return TileJSON document for a COG."""
             route_params = {
@@ -326,7 +385,7 @@ class MosaicTilerFactory(BaseTilerFactory):
                 "x": "{x}",
                 "y": "{y}",
                 "scale": tile_scale,
-                "TileMatrixSetId": tms.identifier,
+                "TileMatrixSetId": TileMatrixSetId,
             }
             if tile_format:
                 route_params["format"] = tile_format.value
@@ -347,11 +406,12 @@ class MosaicTilerFactory(BaseTilerFactory):
             if qs:
                 tiles_url += f"?{urlencode(qs)}"
 
-            with rasterio.Env(**self.gdal_config):
+            with rasterio.Env(**env):
                 with self.reader(
                     src_path,
                     reader=self.dataset_reader,
-                    **self.backend_options,
+                    reader_options={**reader_params},
+                    **backend_params,
                 ) as src_dst:
                     center = list(src_dst.mosaic_def.center)
                     if minzoom is not None:
@@ -364,6 +424,67 @@ class MosaicTilerFactory(BaseTilerFactory):
                         "tiles": [tiles_url],
                     }
 
+    def map_viewer(self):  # noqa: C901
+        """Register /map endpoint."""
+
+        @self.router.get("/{TileMatrixSetId}/map", response_class=HTMLResponse)
+        @self.router.get("/map", response_class=HTMLResponse)
+        def map_viewer(
+            request: Request,
+            src_path=Depends(self.path_dependency),
+            TileMatrixSetId: Literal["WebMercatorQuad"] = Query(
+                "WebMercatorQuad",
+                description="TileMatrixSet Name (default: 'WebMercatorQuad')",
+            ),  # noqa
+            tile_format: Optional[ImageType] = Query(
+                None, description="Output image type. Default is auto."
+            ),
+            tile_scale: int = Query(
+                1, gt=0, lt=4, description="Tile size scale. 1=256x256, 2=512x512..."
+            ),
+            minzoom: Optional[int] = Query(
+                None, description="Overwrite default minzoom."
+            ),
+            maxzoom: Optional[int] = Query(
+                None, description="Overwrite default maxzoom."
+            ),
+            layer_params=Depends(self.layer_dependency),  # noqa
+            dataset_params=Depends(self.dataset_dependency),  # noqa
+            pixel_selection=Depends(self.pixel_selection_dependency),  # noqa
+            buffer: Optional[float] = Query(  # noqa
+                None,
+                gt=0,
+                title="Tile buffer.",
+                description="Buffer on each side of the given tile. It must be a multiple of `0.5`. Output **tilesize** will be expanded to `tilesize + 2 * tile_buffer` (e.g 0.5 = 257x257, 1.0 = 258x258).",
+            ),
+            rescale: Optional[List[Tuple[float, ...]]] = Depends(
+                RescalingParams
+            ),  # noqa
+            color_formula: Optional[str] = Query(  # noqa
+                None,
+                title="Color Formula",
+                description="rio-color formula (info: https://github.com/mapbox/rio-color)",
+            ),
+            colormap=Depends(self.colormap_dependency),  # noqa
+            render_params=Depends(self.render_dependency),  # noqa
+            backend_params=Depends(self.backend_dependency),  # noqa
+            reader_params=Depends(self.reader_dependency),  # noqa
+            env=Depends(self.environment_dependency),  # noqa
+        ):
+            """Return TileJSON document for a dataset."""
+            tilejson_url = self.url_for(request, "tilejson")
+            if request.query_params._list:
+                tilejson_url += f"?{urlencode(request.query_params._list)}"
+
+            return templates.TemplateResponse(
+                name="index.html",
+                context={
+                    "request": request,
+                    "tilejson_endpoint": tilejson_url,
+                },
+                media_type="text/html",
+            )
+
     def wmts(self):  # noqa: C901
         """Add wmts endpoint."""
 
@@ -373,7 +494,10 @@ class MosaicTilerFactory(BaseTilerFactory):
         )
         def wmts(
             request: Request,
-            tms: TileMatrixSet = Depends(self.tms_dependency),
+            TileMatrixSetId: Literal[tuple(self.supported_tms.list())] = Query(
+                "WebMercatorQuad",
+                description="TileMatrixSet Name (default: 'WebMercatorQuad')",
+            ),  # noqa
             src_path=Depends(self.path_dependency),
             tile_format: ImageType = Query(
                 ImageType.png, description="Output image type. Default is png."
@@ -389,12 +513,27 @@ class MosaicTilerFactory(BaseTilerFactory):
             ),
             layer_params=Depends(self.layer_dependency),  # noqa
             dataset_params=Depends(self.dataset_dependency),  # noqa
-            pixel_selection: PixelSelectionMethod = Query(
-                PixelSelectionMethod.first, description="Pixel selection method."
+            pixel_selection=Depends(self.pixel_selection_dependency),  # noqa
+            buffer: Optional[float] = Query(  # noqa
+                None,
+                gt=0,
+                title="Tile buffer.",
+                description="Buffer on each side of the given tile. It must be a multiple of `0.5`. Output **tilesize** will be expanded to `tilesize + 2 * tile_buffer` (e.g 0.5 = 257x257, 1.0 = 258x258).",
+            ),
+            post_process=Depends(self.process_dependency),  # noqa
+            rescale: Optional[List[Tuple[float, ...]]] = Depends(
+                RescalingParams
             ),  # noqa
-            postprocess_params=Depends(self.process_dependency),  # noqa
+            color_formula: Optional[str] = Query(  # noqa
+                None,
+                title="Color Formula",
+                description="rio-color formula (info: https://github.com/mapbox/rio-color)",
+            ),
             colormap=Depends(self.colormap_dependency),  # noqa
             render_params=Depends(self.render_dependency),  # noqa
+            backend_params=Depends(self.backend_dependency),
+            reader_params=Depends(self.reader_dependency),
+            env=Depends(self.environment_dependency),
         ):
             """OGC WMTS endpoint."""
             route_params = {
@@ -403,7 +542,7 @@ class MosaicTilerFactory(BaseTilerFactory):
                 "y": "{TileRow}",
                 "scale": tile_scale,
                 "format": tile_format.value,
-                "TileMatrixSetId": tms.identifier,
+                "TileMatrixSetId": TileMatrixSetId,
             }
             tiles_url = self.url_for(request, "tile", **route_params)
 
@@ -424,11 +563,13 @@ class MosaicTilerFactory(BaseTilerFactory):
             if qs:
                 tiles_url += f"?{urlencode(qs)}"
 
-            with rasterio.Env(**self.gdal_config):
+            tms = self.supported_tms.get(TileMatrixSetId)
+            with rasterio.Env(**env):
                 with self.reader(
                     src_path,
                     reader=self.dataset_reader,
-                    **self.backend_options,
+                    reader_options={**reader_params},
+                    **backend_params,
                 ) as src_dst:
                     bounds = src_dst.geographic_bounds
                     minzoom = minzoom if minzoom is not None else src_dst.minzoom
@@ -483,35 +624,34 @@ class MosaicTilerFactory(BaseTilerFactory):
             src_path=Depends(self.path_dependency),
             layer_params=Depends(self.layer_dependency),
             dataset_params=Depends(self.dataset_dependency),
+            backend_params=Depends(self.backend_dependency),
+            reader_params=Depends(self.reader_dependency),
+            env=Depends(self.environment_dependency),
         ):
             """Get Point value for a Mosaic."""
-            timings = []
             threads = int(os.getenv("MOSAIC_CONCURRENCY", MAX_THREADS))
 
-            with Timer() as t:
-                with rasterio.Env(**self.gdal_config):
-                    with self.reader(
-                        src_path,
-                        reader=self.dataset_reader,
-                        **self.backend_options,
-                    ) as src_dst:
-                        mosaic_read = t.from_start
-                        timings.append(("mosaicread", round(mosaic_read * 1000, 2)))
-                        values = src_dst.point(
-                            lon,
-                            lat,
-                            threads=threads,
-                            **layer_params,
-                            **dataset_params,
-                        )
-            timings.append(("dataread", round((t.elapsed - mosaic_read) * 1000, 2)))
+            with rasterio.Env(**env):
+                with self.reader(
+                    src_path,
+                    reader=self.dataset_reader,
+                    reader_options={**reader_params},
+                    **backend_params,
+                ) as src_dst:
+                    values = src_dst.point(
+                        lon,
+                        lat,
+                        threads=threads,
+                        **layer_params,
+                        **dataset_params,
+                    )
 
-            if OptionalHeader.server_timing in self.optional_headers:
-                response.headers["Server-Timing"] = ", ".join(
-                    [f"{name};dur={time}" for (name, time) in timings]
-                )
-
-            return {"coordinates": [lon, lat], "values": values}
+            return {
+                "coordinates": [lon, lat],
+                "values": [
+                    (src, pts.data.tolist(), pts.band_names) for src, pts in values
+                ],
+            }
 
     def validate(self):
         """Register /validate endpoint."""
@@ -534,13 +674,17 @@ class MosaicTilerFactory(BaseTilerFactory):
             miny: float = Query(None, description="Bottom of bounding box"),
             maxx: float = Query(None, description="Right side of bounding box"),
             maxy: float = Query(None, description="Top of bounding box"),
+            backend_params=Depends(self.backend_dependency),
+            reader_params=Depends(self.reader_dependency),
+            env=Depends(self.environment_dependency),
         ):
             """Return a list of assets which overlap a bounding box"""
-            with rasterio.Env(**self.gdal_config):
+            with rasterio.Env(**env):
                 with self.reader(
                     src_path,
                     reader=self.dataset_reader,
-                    **self.backend_options,
+                    reader_options={**reader_params},
+                    **backend_params,
                 ) as src_dst:
                     return src_dst.assets_for_bbox(minx, miny, maxx, maxy)
 
@@ -552,13 +696,17 @@ class MosaicTilerFactory(BaseTilerFactory):
             src_path=Depends(self.path_dependency),
             lng: float = Query(None, description="Longitude"),
             lat: float = Query(None, description="Latitude"),
+            backend_params=Depends(self.backend_dependency),
+            reader_params=Depends(self.reader_dependency),
+            env=Depends(self.environment_dependency),
         ):
             """Return a list of assets which overlap a point"""
-            with rasterio.Env(**self.gdal_config):
+            with rasterio.Env(**env):
                 with self.reader(
                     src_path,
                     reader=self.dataset_reader,
-                    **self.backend_options,
+                    reader_options={**reader_params},
+                    **backend_params,
                 ) as src_dst:
                     return src_dst.assets_for_point(lng, lat)
 
@@ -571,12 +719,16 @@ class MosaicTilerFactory(BaseTilerFactory):
             x: int = Path(..., description="Mercator tiles's column"),
             y: int = Path(..., description="Mercator tiles's row"),
             src_path=Depends(self.path_dependency),
+            backend_params=Depends(self.backend_dependency),
+            reader_params=Depends(self.reader_dependency),
+            env=Depends(self.environment_dependency),
         ):
             """Return a list of assets which overlap a given tile"""
-            with rasterio.Env(**self.gdal_config):
+            with rasterio.Env(**env):
                 with self.reader(
                     src_path,
                     reader=self.dataset_reader,
-                    **self.backend_options,
+                    reader_options={**reader_params},
+                    **backend_params,
                 ) as src_dst:
                     return src_dst.assets_for_tile(x, y, z)
